<?php
/**
 * Core functions
 *
 * @package BP_Friendship_Restrictions
 */

// Do not allow direct access over web.
use BP_Friendship_Restrictions\Core\Restrictions_Criteria;

defined( 'ABSPATH' ) || exit;

/**
 * Get Default settings.
 *
 * @return array
 */
function bpf_restrictions_get_default_settings() {

	$defaults = array(
		'restrict_receiver'                 => 0,
		'receiver_roles'                    => array( 'all' => 'all' ),
		'invalid_receiver_message'          => __( 'The requested user can not accept new friendship requests.', 'buddypress-friendship-restrictions' ),
		'restrict_sender'                   => 0,
		'sender_roles'                      => array( 'all' => 'all' ),
		'sender_member_types'               => array( 'all' => 'all' ),
		'invalid_sender_message'            => __( 'You are not allowed to send friend requests.', 'buddypress-friendship-restrictions' ),
		'throttle_request'                  => 0,
		'request_threshold'                 => 5,
		'request_throttle_duration'         => 5, // minutes.
		'request_threshold_reached_message' => __( 'You are going too quick. Please be tender.', 'buddypress-friendship-restrictions' ),

		'restrict_friends_count'                     => 0,
		'allowed_friends_count'                      => 10,
		'friends_count_criteria'                     => 'friendship',
		'friends_limit_reached_sender_self_message'  => __( 'You have reached the maximum number of allowed friends.', 'buddypress-friendship-restrictions' ),
		'friends_limit_reached_sender_other_message' => __( 'Limit reached for the user.', 'buddypress-friendship-restrictions' ),

		'friends_limit_reached_acceptor_self_message'  => __( 'You have reached the maximum number of allowed friends.', 'buddypress-friendship-restrictions' ),
		'friends_limit_reached_acceptor_other_message' => __( 'Limit reached for the user.', 'buddypress-friendship-restrictions' ),

		'disable_unfriending'        => 0,
		'unfriending_error_message'  => __( 'Friendship could not be cancelled.', 'buddypress-friendship-restrictions' ),
		'restrict_by'                => '',
		'multi_restriction_criteria' => 'max',

	);

	return (array) apply_filters( 'bp_friendship_restrictions_default_settings', $defaults );
}

/**
 * Get saved settings.
 *
 * @param string $option_key option key value is required.
 *
 * @return mixed
 */
function bpf_restrictions_get_option( $option_key ) {

	$defaults = bpf_restrictions_get_default_settings();
	$settings = get_option( 'bp_friendship_restrictions_settings', $defaults );

	return isset( $settings[ $option_key ] ) ? $settings[ $option_key ] : null;
}

/**
 * Is user a valid sender(has allowed sending roles and is able to send request to friend user id).
 *
 * @param int $user_id user id.
 * @param int $friend_id friend user id.
 *
 * @return bool
 */
function bpf_restrictions_user_is_valid_sender( $user_id, $friend_id ) {

	$restricted = bpf_restrictions_get_option( 'restrict_sender' );

	if ( ! $restricted ) {
		$is_valid = true;
	} else {
		$sender_roles = (array) bpf_restrictions_get_option( 'sender_roles' );
		// default is_valid.
		// everyone is allowed.
		if ( in_array( 'all', $sender_roles ) || is_super_admin( $user_id ) ) {
			$is_valid = true;
		} else {
			$is_valid = bpf_restrictions_user_has_roles_in( $user_id, $sender_roles );
		}
	}

	return (bool) apply_filters( 'bpf_restrictions_user_is_valid_sender', Restrictions_Criteria::get()->is_valid_sender( $user_id, $friend_id, $is_valid ), $is_valid, $user_id, $friend_id );
}

/**
 * Is user a valid receiver(has allowed sending roles).
 *
 * @param int $receiver_user_id user id.
 * @param int $sender_id sender user id.
 *
 * @return bool
 */
function bpf_restrictions_user_is_valid_receiver( $receiver_user_id, $sender_id ) {

	$restricted = bpf_restrictions_get_option( 'restrict_receiver' );

	if ( ! $restricted ) {
		$is_valid = true;
	} else {
		$receiver_roles = (array) bpf_restrictions_get_option( 'receiver_roles' );
		// everyone is allowed.
		if ( in_array( 'all', $receiver_roles ) || is_super_admin( $receiver_user_id ) ) {
			$is_valid = true;
		} else {
			$is_valid = bpf_restrictions_user_has_roles_in( $receiver_user_id, $receiver_roles );
		}
	}

	return (bool) apply_filters( 'bpf_restrictions_user_is_valid_receiver', Restrictions_Criteria::get()->is_valid_receiver( $receiver_user_id, $sender_id, $is_valid ), $is_valid, $receiver_user_id, $sender_id );
}

/**
 * Check if user has reached threshold.
 *
 * @param int $user_id user id.
 *
 * @return bool
 */
function bpf_restrictions_user_reached_request_threshold( $user_id ) {

	$throttled = bpf_restrictions_get_option( 'throttle_request' );
	if ( ! $throttled ) {
		$has_reached = false;
	} else {
		if ( is_super_admin( $user_id ) ) {
			$has_reached = false;
		} else {
			$has_reached = bpf_restrictions_get_pending_sent_request_count_for_duration( $user_id, bpf_restrictions_get_option( 'request_throttle_duration' ) ) >= bpf_restrictions_get_option( 'request_threshold' );
		}
	}

	return (bool) apply_filters( 'bpf_restrictions_user_reached_request_threshold', Restrictions_Criteria::get()->has_reached_request_threshold( $user_id, $has_reached ), $has_reached, $user_id );
}

/**
 * Check if friendship limit reached for the given user.
 *
 * @param int $user_id user id.
 *
 * @return bool
 */
function bpf_restrictions_user_reached_friendship_limit( $user_id ) {

	$restricted = bpf_restrictions_get_option( 'restrict_friends_count' );
	if ( ! $restricted ) {
		$has_reached = false;
	} else {
		// since it is restricted, assume has reached.
		$has_reached = true;

		$allowed_friends_count = bpf_restrictions_get_option( 'allowed_friends_count' );
		// User is super admin, there is no limit for them.
		if ( is_super_admin( $user_id ) ) {
			$has_reached = false;
		} elseif ( $allowed_friends_count <= 0 ) {
			$has_reached = true;
		} else {

			$total_friends = friends_get_friend_count_for_user( $user_id );

			if ( 'request' === bpf_restrictions_get_option( 'friends_count_criteria' ) ) {
				$requests_by_user_count = bpf_restrictions_get_pending_request_count( $user_id );

				$total_friends = $total_friends + $requests_by_user_count;
			}

			if ( $total_friends < $allowed_friends_count ) {
				$has_reached = false;
			}
		}
	}

	return (bool) apply_filters( 'bpf_restrictions_user_reached_friendship_limit', Restrictions_Criteria::get()->has_reached_friendship_limit( $user_id, $has_reached ), $has_reached, $user_id );
}

/**
 * Get total pending request count for user(includes request sent+request received)
 *
 * @param int $user_id User id.
 *
 * @todo cache.
 *
 * @return int
 */
function bpf_restrictions_get_pending_request_count( $user_id ) {

	global $wpdb;

	$bp         = buddypress();
	$oc_sql     = 'AND is_confirmed = 0';
	$friend_sql = $wpdb->prepare( " WHERE ( initiator_user_id = %d OR friend_user_id = %d )", $user_id, $user_id );

	$friends_count = $wpdb->get_var( "SELECT COUNT('*') FROM {$bp->friends->table_name} {$friend_sql} {$oc_sql} ORDER BY date_created DESC" );

	return absint( $friends_count );
}

/**
 * Get count for sent request pending.
 *
 * @param int $user_id User id.
 * @param int $duration time in minutes.
 *
 * @todo cache.
 *
 * @return int
 */
function bpf_restrictions_get_pending_sent_request_count_for_duration( $user_id, $duration ) {

	global $wpdb;
	$bp = buddypress();

	$oc_sql     = 'AND is_confirmed = 0';
	$friend_sql = $wpdb->prepare( " WHERE initiator_user_id = %d ", $user_id );
	$time_sql   = $wpdb->prepare( "AND DATE_ADD(date_created, INTERVAL %d MINUTE ) >= UTC_TIMESTAMP()", $duration );

	$friends_count = $wpdb->get_var( "SELECT COUNT('*') FROM {$bp->friends->table_name} {$friend_sql} {$oc_sql} {$time_sql} " );

	return absint( $friends_count );
}

/**
 * Check if the given user has atleast one role in teh given roles array.
 *
 * @param int   $user_id user id.
 * @param array $roles roles to be checked.
 *
 * @return bool
 */
function bpf_restrictions_user_has_roles_in( $user_id, $roles ) {
	$roles = (array) $roles;
	// is everyone allowed?
	if ( in_array( 'all', $roles ) ) {
		return true;
	} elseif ( empty( $roles ) ) {
		return false;// none allowed.
	}

	$user = get_user_by( 'id', $user_id );
	if ( empty( $user ) ) {
		return false;
	}

	$user_roles = $user->roles;

	if ( empty( $user_roles ) ) {
		return false;
	}

	// check if the given user has at least one role in the roles?
	if ( ! array_intersect( $roles, $user_roles ) ) {
		return false;
	}

	return true;
}


/**
 * Check if the given user has atleast one role in teh given roles array.
 *
 * @param int   $user_id user id.
 * @param array $member_types roles to be checked.
 *
 * @return bool
 */
function bpf_restrictions_user_has_member_type_in( $user_id, $member_types ) {

	$member_types = (array) $member_types;

	// is everyone allowed?
	if ( in_array( 'all', $member_types ) ) {
		return true;
	} elseif ( empty( $member_types ) ) {
		return false;// none allowed.
	}

	$user = get_user_by( 'id', $user_id );

	if ( empty( $user ) ) {
		return false;
	}

	$user_member_types = bp_get_member_type( $user_id, false );

	if ( empty( $user_member_types ) ) {
		return false;
	}

	// check if the given user has at least one role in the roles?
	if ( ! array_intersect( $member_types, $user_member_types ) ) {
		return false;
	}

	return true;
}
/**
 * Check if un-friend feature enabled or disabled by admin
 *
 * @return bool
 */
function bpf_restrictions_is_unfriending_disabled() {
	return bpf_restrictions_get_option( 'disable_unfriending' );
}

/**
 * Get all roles
 *
 * @return array
 */
function bpf_restrictions_get_roles() {
	$user_roles        = array();
	$user_roles['all'] = __( 'All Members', 'buddypress-friendship-restrictions' );

	$roles = get_editable_roles();

	foreach ( $roles as $role => $role_info ) {
		$user_roles[ $role ] = $role_info['name'];
	}

	return $user_roles;
}
