<?php
/**
 * Intercept friendship actions and disable if needed.
 *
 * @package    BuddyPress Friendship Restrictions
 * @subpackage Handlers
 * @copyright  Copyright (c) 2018, BuddyDev
 *
 * @author Ravi Sharma, Brajesh Singh
 */

namespace BP_Friendship_Restrictions\Handlers;

// Exit if files accessed directly over web.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Class Actions_Handler
 */
class Actions_Handler {

	/**
	 * Boot class
	 */
	public static function boot() {
		$self = new self();
		$self->setup();
	}

	/**
	 * The constructor.
	 */
	private function setup() {
		add_action( 'bp_actions', array( $this, 'gate_add_friend' ), 0 );
		add_action( 'bp_actions', array( $this, 'gate_remove_friend' ), 0 );

		// check on accept friend.
		add_action( 'bp_screens', array( $this, 'check_limit_on_accept' ), 0 );
	}

	/**
	 * Check and allow to add friend
	 */
	public function gate_add_friend() {

		// No Restriction for super admin.
		if ( is_super_admin() ) {
			return;
		}

		if ( ! bp_is_friends_component() || ! bp_is_current_action( 'add-friend' ) ) {
			return;
		}

		if ( ! $potential_friend_id = (int) bp_action_variable( 0 ) ) {
			return; // Not of any interest to us as potential friend is not specified.
		}

		if ( bp_loggedin_user_id() == $potential_friend_id ) {
			return; // Invalid request.
		}

		$logged_id = bp_loggedin_user_id();

		if (! bpf_restrictions_user_is_valid_sender( $logged_id, $potential_friend_id ) ) {
			// Restricted send for the user.
			bp_core_add_message( bpf_restrictions_get_option( 'invalid_sender_message' ), 'error' );
			bp_core_redirect( wp_get_referer() );
		} elseif ( ! bpf_restrictions_user_is_valid_receiver( $potential_friend_id, $logged_id ) ) {
			bp_core_add_message( bpf_restrictions_get_option( 'invalid_receiver_message' ), 'error' );
			bp_core_redirect( wp_get_referer() );
		} elseif ( bpf_restrictions_user_reached_request_threshold( $logged_id ) ) {
			// if throttling is enabled and the logged user has reached limit, show error.
			bp_core_add_message( bpf_restrictions_get_option( 'request_threshold_reached_message' ), 'error' );
			bp_core_redirect( wp_get_referer() );
		}

		// if we are here, friendship limit is enabled, let us check for both.
		if ( bpf_restrictions_user_reached_friendship_limit( $logged_id ) ) {
			bp_core_add_message( bpf_restrictions_get_option( 'friends_limit_reached_sender_self_message' ), 'error' );
			bp_core_redirect( wp_get_referer() );
		} elseif ( bpf_restrictions_user_reached_friendship_limit( $potential_friend_id ) ) {
			bp_core_add_message( bpf_restrictions_get_option( 'friends_limit_reached_sender_other_message' ), 'error' );

			bp_core_redirect( wp_get_referer() );
		}

		// All good, if we are here.
	}

	/**
	 * Remove friend
	 */
	public function gate_remove_friend() {

		// No restrictions for site admins.
		if ( is_super_admin() ) {
			return;
		}

		// Not our request.
		if ( ! bp_is_friends_component() || ! bp_is_current_action( 'remove-friend' ) ) {
			return;
		}

		// Invalid request.
		if ( ! $potential_friend_id = (int) bp_action_variable( 0 ) ) {
			return;
		}

		// if un-friending is disabled, disable it.
		if ( bpf_restrictions_is_unfriending_disabled() && 'is_friend' == bp_is_friend( $potential_friend_id ) ) {

			bp_core_add_message( bpf_restrictions_get_option( 'unfriending_error_message' ), 'error' );

			bp_core_redirect( wp_get_referer() );

		}
	}

	/**
	 * Check on accept
	 */
	public function check_limit_on_accept() {

		if ( is_super_admin() ) {
			return;
		}

		if ( ! bp_is_friends_component() || ! bp_is_action_variable( 'accept', 0 ) || ! is_numeric( bp_action_variable( 1 ) ) ) {
			return;
		}

		// Check the nonce.
		check_admin_referer( 'friends_accept_friendship' );

		$fid = absint( bp_action_variable( 1 ) );

		$friendship = new \BP_Friends_Friendship( $fid, false, false );

		$initiator_user_id = $friendship->initiator_user_id;
		$friend_user_id    = $friendship->friend_user_id;
		// check.
		if ( bpf_restrictions_user_reached_friendship_limit( $friend_user_id ) ) {
			bp_core_add_message( bpf_restrictions_get_option( 'friends_limit_reached_acceptor_self_message' ), 'error' );

			bp_core_redirect( wp_get_referer() );
		} elseif ( bpf_restrictions_user_reached_friendship_limit( $initiator_user_id ) ) {
			bp_core_add_message( bpf_restrictions_get_option( 'friends_limit_reached_acceptor_other_message' ), 'error' );

			bp_core_redirect( wp_get_referer() );
		}

	}

}
